'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.installApk = exports.getDevices = exports.getAndroidAvds = exports.launchAndroidActivity = exports.buildAndInstallApp = exports.androidGetBootAnimProp = exports.waitForAndroidDevice = exports.runAndroidUsingAvdImage = exports.runAndroid = exports.runAndroidApk = exports.runAndroidProject = undefined;

// ==============================================================================
// Misc utilities
// ==============================================================================

//
// Returns a promise that will get resolved after a given delay (in ms)
var delay = function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(ms) {
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            return _context.abrupt('return', new Promise(function (resolve, reject) {
              setTimeout(function () {
                resolve();
              }, ms);
            }));

          case 1:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  return function delay(_x) {
    return _ref.apply(this, arguments);
  };
}();

// ==============================================================================
// Core android stuff
// ==============================================================================

//
// Build and run a project on an Android emulator or connected device
// The `devDebug` variant will be built and launched on an emulator selected by
// the user (this function prompts the user with a list of available avd to choose from)
//
// Assumptions :
// - devDebug variant exists in the project
// Params :
// - projectPath : Absolute or relative path to the root of the Android projectPath
// - packageName : Name of the package containing the application
// Options :
// - activityName : The name of the Activity to start (default "MainActivity")


var runAndroidProject = exports.runAndroidProject = function () {
  var _ref2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(_ref3) {
    var projectPath = _ref3.projectPath,
        packageName = _ref3.packageName,
        _ref3$activityName = _ref3.activityName,
        activityName = _ref3$activityName === undefined ? 'MainActivity' : _ref3$activityName;
    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            return _context2.abrupt('return', runAndroid({
              projectPath,
              packageName,
              activityName
            }));

          case 1:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));

  return function runAndroidProject(_x2) {
    return _ref2.apply(this, arguments);
  };
}();

//
// Run an android APK on an Android emulator or connected device
//
// Params :
// - apkPath : Absolute or relative path to the APK
// - packageName : Name of the package containing the application
// Options :
// - activityName : The name of the Activity to start (default "MainActivity")


var runAndroidApk = exports.runAndroidApk = function () {
  var _ref4 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3(_ref5) {
    var apkPath = _ref5.apkPath,
        packageName = _ref5.packageName,
        _ref5$activityName = _ref5.activityName,
        activityName = _ref5$activityName === undefined ? 'MainActivity' : _ref5$activityName;
    return regeneratorRuntime.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            return _context3.abrupt('return', runAndroid({
              apkPath,
              packageName,
              activityName
            }));

          case 1:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, this);
  }));

  return function runAndroidApk(_x3) {
    return _ref4.apply(this, arguments);
  };
}();

var runAndroid = exports.runAndroid = function () {
  var _ref6 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee4(_ref7) {
    var packageName = _ref7.packageName,
        activityName = _ref7.activityName,
        apkPath = _ref7.apkPath,
        projectPath = _ref7.projectPath;
    var devices, avdImageName;
    return regeneratorRuntime.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            _context4.next = 2;
            return getDevices();

          case 2:
            devices = _context4.sent;

            if (!(devices.length === 1)) {
              _context4.next = 9;
              break;
            }

            log.debug(devices[0].split('\t')[0], ' is running ...');
            _context4.next = 7;
            return installAndLaunchApp({
              projectPath,
              packageName,
              apkPath,
              activityName
            });

          case 7:
            _context4.next = 17;
            break;

          case 9:
            if (!(devices.length > 1)) {
              _context4.next = 13;
              break;
            }

            throw new Error('More than one device/emulator is running !');

          case 13:
            _context4.next = 15;
            return askUserToSelectAvdEmulator();

          case 15:
            avdImageName = _context4.sent;
            return _context4.abrupt('return', runAndroidUsingAvdImage({
              projectPath,
              packageName,
              avdImageName,
              apkPath,
              activityName
            }));

          case 17:
          case 'end':
            return _context4.stop();
        }
      }
    }, _callee4, this);
  }));

  return function runAndroid(_x4) {
    return _ref6.apply(this, arguments);
  };
}();

var askUserToSelectAvdEmulator = function () {
  var _ref8 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee5() {
    var avdImageNames, emulatorConfig, emulatorName, avdIndex, _ref9, avdImageName;

    return regeneratorRuntime.wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            _context5.next = 2;
            return getAndroidAvds();

          case 2:
            avdImageNames = _context5.sent;
            emulatorConfig = _config2.default.getValue('emulatorConfig');

            // Check if user has set the usePreviousEmulator flag to true

            if (!(avdImageNames && emulatorConfig.android.usePreviousEmulator)) {
              _context5.next = 9;
              break;
            }

            // Get the name of previously used emulator
            emulatorName = emulatorConfig.android.emulatorName;
            // Check if avd image still exists

            avdIndex = avdImageNames.indexOf(emulatorName);

            if (!(avdIndex > -1)) {
              _context5.next = 9;
              break;
            }

            return _context5.abrupt('return', `${avdImageNames[avdIndex]}`);

          case 9:
            _context5.next = 11;
            return _inquirer2.default.prompt([{
              type: 'list',
              name: 'avdImageName',
              message: 'Choose Android emulator image',
              choices: avdImageNames
            }]);

          case 11:
            _ref9 = _context5.sent;
            avdImageName = _ref9.avdImageName;


            // Update the emulatorConfig
            if (emulatorConfig) {
              emulatorConfig.android.emulatorName = avdImageName;
              _config2.default.setValue('emulatorConfig', emulatorConfig);
            }
            return _context5.abrupt('return', `${avdImageName}`);

          case 15:
          case 'end':
            return _context5.stop();
        }
      }
    }, _callee5, this);
  }));

  return function askUserToSelectAvdEmulator() {
    return _ref8.apply(this, arguments);
  };
}();

// Does the job of actually running the app
// It orchestrates a few tasks to actually get the job done
// Params :
// - projectPath : Absolute or relative path to the root of the Android projectPath
// - packageName : name of the package containing the application
// - avdImageName : name of the avd image to use (emulator image)


var runAndroidUsingAvdImage = exports.runAndroidUsingAvdImage = function () {
  var _ref10 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee6(_ref11) {
    var projectPath = _ref11.projectPath,
        packageName = _ref11.packageName,
        avdImageName = _ref11.avdImageName,
        activityName = _ref11.activityName;
    return regeneratorRuntime.wrap(function _callee6$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:
            // https://issuetracker.google.com/issues/37137213
            (0, _childProcess.spawnp)(androidEmulatorPath(), ['-avd', avdImageName], { detached: true });

            _context6.next = 3;
            return (0, _spin2.default)('Waiting for device to start', waitForAndroidDevice());

          case 3:
            _context6.next = 5;
            return installAndLaunchApp({ projectPath, packageName, activityName });

          case 5:
          case 'end':
            return _context6.stop();
        }
      }
    }, _callee6, this);
  }));

  return function runAndroidUsingAvdImage(_x5) {
    return _ref10.apply(this, arguments);
  };
}();

// Does the job of installing and running the app
// It orchestrates a few tasks to actually get the job done
// Params :
// - projectPath : Absolute or relative path to the root of the Android projectPath
// - packageName : name of the package containing the application


var installAndLaunchApp = function () {
  var _ref12 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee7(_ref13) {
    var projectPath = _ref13.projectPath,
        apkPath = _ref13.apkPath,
        packageName = _ref13.packageName,
        activityName = _ref13.activityName;
    return regeneratorRuntime.wrap(function _callee7$(_context7) {
      while (1) {
        switch (_context7.prev = _context7.next) {
          case 0:
            if (!projectPath) {
              _context7.next = 5;
              break;
            }

            _context7.next = 3;
            return (0, _spin2.default)('Building and installing application', buildAndInstallApp(projectPath));

          case 3:
            _context7.next = 8;
            break;

          case 5:
            if (!apkPath) {
              _context7.next = 8;
              break;
            }

            _context7.next = 8;
            return (0, _spin2.default)('Installing APK', installApk(apkPath));

          case 8:
            _context7.next = 10;
            return (0, _spin2.default)('Launching Android Application', Promise.resolve());

          case 10:
            launchAndroidActivityDetached(packageName, activityName);

          case 11:
          case 'end':
            return _context7.stop();
        }
      }
    }, _callee7, this);
  }));

  return function installAndLaunchApp(_x6) {
    return _ref12.apply(this, arguments);
  };
}();

// Utility method that basically completes whenever the android device is ready
// It check device readiness every 2 sec (poll way)


var waitForAndroidDevice = exports.waitForAndroidDevice = function () {
  var _ref14 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee8() {
    var androidBootAnimProp;
    return regeneratorRuntime.wrap(function _callee8$(_context8) {
      while (1) {
        switch (_context8.prev = _context8.next) {
          case 0:
            _context8.next = 2;
            return androidGetBootAnimProp();

          case 2:
            androidBootAnimProp = _context8.sent;

          case 3:
            if (androidBootAnimProp.toString().startsWith('stopped')) {
              _context8.next = 11;
              break;
            }

            _context8.next = 6;
            return delay(2000);

          case 6:
            _context8.next = 8;
            return androidGetBootAnimProp();

          case 8:
            androidBootAnimProp = _context8.sent;
            _context8.next = 3;
            break;

          case 11:
          case 'end':
            return _context8.stop();
        }
      }
    }, _callee8, this);
  }));

  return function waitForAndroidDevice() {
    return _ref14.apply(this, arguments);
  };
}();

// Utility method to know when the prop init.svc.bootanim is there
// which indicates somehow that device is ready to install APK and such


var androidGetBootAnimProp = exports.androidGetBootAnimProp = function () {
  var _ref15 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee9() {
    return regeneratorRuntime.wrap(function _callee9$(_context9) {
      while (1) {
        switch (_context9.prev = _context9.next) {
          case 0:
            return _context9.abrupt('return', (0, _childProcess.execp)(`${androidAdbPath()} wait-for-device shell getprop init.svc.bootanim`));

          case 1:
          case 'end':
            return _context9.stop();
        }
      }
    }, _callee9, this);
  }));

  return function androidGetBootAnimProp() {
    return _ref15.apply(this, arguments);
  };
}();

// Build & install application on the device
// params :
// - projectPath : Absolute or relative path to the root of the Android project
// containing the application


var buildAndInstallApp = exports.buildAndInstallApp = function () {
  var _ref16 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee10(projectPath) {
    var gradlew;
    return regeneratorRuntime.wrap(function _callee10$(_context10) {
      while (1) {
        switch (_context10.prev = _context10.next) {
          case 0:
            _shell2.default.cd(projectPath);
            gradlew = /^win/.test(process.platform) ? 'gradlew' : './gradlew';
            return _context10.abrupt('return', (0, _childProcess.execp)(`${gradlew} installDebug`));

          case 3:
          case 'end':
            return _context10.stop();
        }
      }
    }, _callee10, this);
  }));

  return function buildAndInstallApp(_x7) {
    return _ref16.apply(this, arguments);
  };
}();

// Utility method to launch a specific activity from a given package
// Params :
// - packageName : name of the package containing the application
// - activityName : name of the Activity to launch


var launchAndroidActivity = exports.launchAndroidActivity = function () {
  var _ref17 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee11(packageName, activityName) {
    return regeneratorRuntime.wrap(function _callee11$(_context11) {
      while (1) {
        switch (_context11.prev = _context11.next) {
          case 0:
            return _context11.abrupt('return', (0, _childProcess.execp)(`${androidAdbPath()} shell am start -n ${packageName}/.${activityName}`));

          case 1:
          case 'end':
            return _context11.stop();
        }
      }
    }, _callee11, this);
  }));

  return function launchAndroidActivity(_x8, _x9) {
    return _ref17.apply(this, arguments);
  };
}();

// Utility method to launch a specific activity from a given packager
// Will spawn the command (detached mode)


// Utility method to list all available android avd images (emulator images)
var getAndroidAvds = exports.getAndroidAvds = function () {
  var _ref18 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee12() {
    var stdout;
    return regeneratorRuntime.wrap(function _callee12$(_context12) {
      while (1) {
        switch (_context12.prev = _context12.next) {
          case 0:
            _context12.next = 2;
            return (0, _childProcess.execp)(`${androidEmulatorPath()} -list-avds`);

          case 2:
            stdout = _context12.sent;
            return _context12.abrupt('return', stdout.toString().trim().split('\n'));

          case 4:
          case 'end':
            return _context12.stop();
        }
      }
    }, _callee12, this);
  }));

  return function getAndroidAvds() {
    return _ref18.apply(this, arguments);
  };
}();

// Utility method to query what device instances are connected to the adb server


var getDevices = exports.getDevices = function () {
  var _ref19 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee13() {
    var stdout, stdOutArr;
    return regeneratorRuntime.wrap(function _callee13$(_context13) {
      while (1) {
        switch (_context13.prev = _context13.next) {
          case 0:
            _context13.next = 2;
            return (0, _childProcess.execp)(`${androidAdbPath()} devices`);

          case 2:
            stdout = _context13.sent;

            /*
              stdout for running command  $adb devices
              List of devices attached
              * daemon not running. starting it now at tcp:5037 *
              * daemon started successfully *
            */
            stdOutArr = stdout.toString().trim().split('\n');
            // remove stdout 'List of devices attached' (position 0)
            // and remove stdout related to daemon

            return _context13.abrupt('return', stdOutArr.filter(function (entry, i) {
              return i > 0 && !entry.includes('* daemon');
            }));

          case 5:
          case 'end':
            return _context13.stop();
        }
      }
    }, _callee13, this);
  }));

  return function getDevices() {
    return _ref19.apply(this, arguments);
  };
}();

var installApk = exports.installApk = function () {
  var _ref20 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee14(pathToApk) {
    return regeneratorRuntime.wrap(function _callee14$(_context14) {
      while (1) {
        switch (_context14.prev = _context14.next) {
          case 0:
            return _context14.abrupt('return', (0, _childProcess.execp)(`${androidAdbPath()} install -r ${pathToApk}`));

          case 1:
          case 'end':
            return _context14.stop();
        }
      }
    }, _callee14, this);
  }));

  return function installApk(_x10) {
    return _ref20.apply(this, arguments);
  };
}();

exports.launchAndroidActivityDetached = launchAndroidActivityDetached;
exports.androidAdbPath = androidAdbPath;
exports.androidEmulatorPath = androidEmulatorPath;

var _inquirer = require('inquirer');

var _inquirer2 = _interopRequireDefault(_inquirer);

var _shell = require('./shell');

var _shell2 = _interopRequireDefault(_shell);

var _spin = require('./spin.js');

var _spin2 = _interopRequireDefault(_spin);

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

var _childProcess = require('./childProcess');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function launchAndroidActivityDetached(packageName, activityName) {
  (0, _childProcess.spawnp)(androidAdbPath(), ['shell', 'am', 'start', '-n', `${packageName}/.${activityName}`], { detached: true });
}function androidAdbPath() {
  return process.env.ANDROID_HOME ? `${process.env.ANDROID_HOME}/platform-tools/adb` : 'adb';
}

function androidEmulatorPath() {
  return process.env.ANDROID_HOME ? `${process.env.ANDROID_HOME}/tools/emulator` : 'emulator';
}
//# sourceMappingURL=android.js.map