'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = findNativeDependencies;

var _Dependency = require('./Dependency');

var _Dependency2 = _interopRequireDefault(_Dependency);

var _fsReaddirRecursive = require('fs-readdir-recursive');

var _fsReaddirRecursive2 = _interopRequireDefault(_fsReaddirRecursive);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var NPM_SCOPED_MODULE_RE = /@(.*)\/(.*)/;
var API_PATH_RE = /^win/.test(process.platform) ? /react-native-.+-api\\/ : /react-native-.+-api\//;

function findNativeDependencies(p) {
  var result = [];

  var nativeDependenciesNames = new Set();

  var nodeModulesDirectoriesWithNativeCode = (0, _fsReaddirRecursive2.default)(p).filter(function (a) {
    return a.includes('build.gradle') || a.includes('.pbxproj') || API_PATH_RE.test(a);
  });

  // By convention we only assume react native plugins to be in directories
  // which names are starting with 'react-native' (excluding scope)
  var nativeDepsDirectories = _lodash2.default.filter(nodeModulesDirectoriesWithNativeCode, function (d) {
    return d.includes('react-native') && !/sample|demo|example/i.test(d);
  });

  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = nativeDepsDirectories[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var nativeDepsDirectory = _step.value;

      var pathSegments = nativeDepsDirectory.split(_path2.default.sep);
      if (pathSegments[0].startsWith('@')) {
        nativeDependenciesNames.add(`${pathSegments[0]}/${pathSegments[1]}`);
      } else {
        nativeDependenciesNames.add(pathSegments[0]);
      }
    }

    // Get associated versions
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  var _iteratorNormalCompletion2 = true;
  var _didIteratorError2 = false;
  var _iteratorError2 = undefined;

  try {
    for (var _iterator2 = nativeDependenciesNames[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
      var nativeDependencyName = _step2.value;

      var pathToNativeDependencyPackageJson = _path2.default.join(p, nativeDependencyName, 'package.json');
      var nativeDepPackageJson = require(pathToNativeDependencyPackageJson);
      if (NPM_SCOPED_MODULE_RE.test(nativeDependencyName)) {
        result.push(new _Dependency2.default(NPM_SCOPED_MODULE_RE.exec(nativeDependencyName)[2], {
          scope: NPM_SCOPED_MODULE_RE.exec(nativeDependencyName)[1],
          version: nativeDepPackageJson.version.startsWith('v') ? nativeDepPackageJson.version.slice(1) : nativeDepPackageJson.version
        }));
      } else {
        result.push(new _Dependency2.default(nativeDependencyName, {
          version: nativeDepPackageJson.version.startsWith('v') ? nativeDepPackageJson.version.slice(1) : nativeDepPackageJson.version
        }));
      }
    }
  } catch (err) {
    _didIteratorError2 = true;
    _iteratorError2 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion2 && _iterator2.return) {
        _iterator2.return();
      }
    } finally {
      if (_didIteratorError2) {
        throw _iteratorError2;
      }
    }
  }

  return result;
}
//# sourceMappingURL=findNativeDependencies.js.map